(function () {
  'use strict';

  function MapConfig($stateProvider) {
    $stateProvider

      .state('map', {
        url: "/map/allCars",
        templateUrl: "templates/map.html",
        controller: 'MapCtrl',
        resolve: {
          Markers: function ($http, MetaVera) {

            var serviceUrl = MetaVera.getQueryVariable() == 'dev' ?
                'http://localhost:3030/maps/?format=json' : //if
                'https://reserve.streetcar.ru/maps/?format=json'; //else

            return $http.get(serviceUrl, {cache: true}).then(function (response) {

              var markers = _.filter(response.data[0], function (stack) {
                return stack;
              });

              return markers;
            });
          }
        }
      })

      .state('mapByType', {
        url: "/map/type/:id",
        templateUrl: "templates/map.html",
        controller: 'MapCtrl',
        resolve: {
          Markers: function ($http, $stateParams, MetaVera) {

            var serviceUrl = MetaVera.getQueryVariable() == 'dev' ?
                'http://localhost:3030/maps/?format=json' : //if
                'https://reserve.streetcar.ru/maps/?format=json'; //else

            return $http.get(serviceUrl, {cache: true}).then(function (response) {

              var markers = _.filter(response.data[0], function(stack){
                return stack.vehicle_types == $stateParams['id'];
              });

              return markers;
            });
          }
        }
      })

      .state('mapByVehicle', {
        url: "/map/vehicle/:id",
        templateUrl: "templates/map.html",
        controller: 'MapCtrl',
        resolve: {
          Markers: function ($http, $stateParams, MetaVera) {

            var serviceUrl = MetaVera.getQueryVariable() == 'dev' ?
                'http://localhost:3030/maps/?format=json' : //if
                'https://reserve.streetcar.ru/maps/?format=json'; //else

            return $http.get(serviceUrl, {cache: true}).then(function (response) {

              var markers = _.filter(response.data[0], function(stack){
                return stack.vehicles == $stateParams['id'] + ',';
              });

              return markers;
            });
          }
        }
      })

      .state('mapByVehicles', {
        url: "/map/vehicles?start&end&lat&long&name&car&type",
        templateUrl: "templates/map.html",
        controller: 'MapCtrl',
        resolve: {
          Markers: function ($http, $stateParams, MetaVera) {

            var serviceUrl = MetaVera.getQueryVariable() == 'dev' ?
                'http://localhost:3030/maps/?format=json' : //if
                'https://reserve.streetcar.ru/maps/?format=json'; //else

            return $http.get(serviceUrl, {cache: true}).then(function (response) {
              var markers = _.filter(response.data[0], function(stack){
                stack.startTime = $stateParams['start'];
                stack.endTime = $stateParams['end'];
                if(typeof $stateParams.type == 'string') {
                  if($stateParams.type == stack.vehicle_types) {
                    return _.contains($stateParams['car'] + ',', stack.vehicles);
                  }
                } else {
                  return _.contains($stateParams['car'] + ',', stack.vehicles);
                }
              });
              return markers;
            });

          }
        }
      });
  }

  angular
    .module('app.map', [
      'ui.router',
      'app.services.metavera',
      'app.services.user',
      'app.services.sf',
      'yaMap'
    ])
    .config(MapConfig);

}());
