function openOnMap(long, lat) {
  // If it's an iPhone..
  if ((navigator.platform.indexOf("iPhone") != -1)
    || (navigator.platform.indexOf("iPod") != -1)
    || (navigator.platform.indexOf("iPad") != -1))
    window.open("maps://maps.google.com/maps?daddr=" + lat + "," + long + "&amp;ll=");
  else
    window.open("http://maps.google.com/maps?daddr=" + lat + ", " + long + "&amp;ll=");
};

(function () {
  'use strict';

  function MapCtrl($scope, $interval, $cordovaGeolocation, Markers, $localStorage, sf, $stateParams, templateLayoutFactory, $window) {

    var user = $localStorage.getObject('user');

    function getMarkers(){
      var items = [];
      for(var i = 0; i < Markers.length; i++){
        var splitId = (Markers[i].vehicles).split(',');
        var carName = sf.getCarName(Markers[i].vehicle_types);
        var time = Markers[i].startTime && Markers[i].endTime ? '&start='+ Markers[i].startTime +'&end='+ Markers[i].endTime +'' : '';
        var balloonFooter = user.user ? '<a class="doReservation" href="#/my/newreservation/'+ splitId[0] +'?lat='+Markers[i].latitude+'&long='+Markers[i].longitude+time+'">'+ sf.translate('filter.book_m') +'</a>' : '';
        items.unshift({
          geometry: {
            type: 'Point',
            coordinates: [Number(Markers[i].longitude), Number(Markers[i].latitude)]
          },
          properties: {
            balloonContentHeader: carName,
            balloonContentBody: Markers[i].descr,
            balloonContentFooter: balloonFooter
          },
          options: {
            hideIconOnBalloonOpen: false,
            iconLayout: 'default#image',
            iconImageHref: carName.indexOf('BMW') != -1 ? 'img/pins.png' : 'img/pins_vw.png',
            iconImageSize: [45, 50],
            iconImageOffset: [-17, -49],
            balloonOffset: [0, -35]
          }
        });
      }
      return items;
    }

    $interval(function () {
      $cordovaGeolocation
          .getCurrentPosition({})
          .then(function (position) {
            var lat = position.coords.latitude;
            var long = position.coords.longitude;
            $scope.map.markers.me = {
              geometry: {
                type: 'Point',
                coordinates: [long, lat]
              }
            };
          }, function (err) {
            // error
          });
    }, 2000);

    $scope.centerOnMe = function() {
      console.log("Centering");

      if ( $scope.map.markers.me && $scope.map.markers.me.geometry.coordinates ) {
        $scope.map.center = $scope.map.markers.me.geometry.coordinates;
      }
    };

    if($stateParams.lat && $stateParams.long){
      $scope.place = {
        geometry: {
          type: 'Point',
          coordinates: [$stateParams.long, Number($stateParams.lat)]
        },
        properties: {
          balloonContentHeader: sf.translate('filter.places.title'),
          balloonContentBody: $stateParams.name
        },
        options: {
          hideIconOnBalloonOpen: false,
          iconLayout: 'default#image',
          iconImageHref: 'img/pin_green.png',
          iconImageSize: [50, 50],
          iconImageOffset: [-15, -49],
          balloonOffset: [0, -35]
        }
      }
    }

    $scope.map = {
      markers: getMarkers(),
      center: [30.315868, 59.939095]
    };

    $scope.geoOptions = {
      layout:'buttonTemplate',
      selectOnClick: false,
      maxWidth: 45,
      position: {
        bottom: 55,
        right: 10
      }
    };

    $scope.zoomOptions = {
      layout:'zoomTemplate',
      size: 'small',
      position: {
        top: ($window.innerHeight - 110) / 2,
        right: 10
      }
    };

    $scope.mapHeight = $window.innerHeight;

    $scope.zoomOver = {
      build: function () {
        var ZoomLayout = templateLayoutFactory.get('zoomTemplate');
        ZoomLayout.superclass.build.call(this);

        this.zoomInCallback = ymaps.util.bind(this.zoomIn, this);
        this.zoomOutCallback = ymaps.util.bind(this.zoomOut, this);

        angular.element(document.getElementById('zoom-in')).bind('click', this.zoomInCallback);
        angular.element(document.getElementById('zoom-out')).bind('click', this.zoomOutCallback);
      },

      clear: function () {
        angular.element(document.getElementById('zoom-in')).unbind('click', this.zoomInCallback);
        angular.element(document.getElementById('zoom-out')).unbind('click', this.zoomOutCallback);

        var ZoomLayout = templateLayoutFactory.get('zoomTemplate');
        ZoomLayout.superclass.clear.call(this);
      },

      zoomIn: function () {
        var map = this.getData().control.getMap();
        this.events.fire('zoomchange', {
          oldZoom: map.getZoom(),
          newZoom: map.getZoom() + 1
        });
      },

      zoomOut: function () {
        var map = this.getData().control.getMap();
        this.events.fire('zoomchange', {
          oldZoom: map.getZoom(),
          newZoom: map.getZoom() - 1
        });
      }
    };

    $scope.openOnMap = openOnMap;

  }

  angular.module('app.map')
    .controller('MapCtrl', MapCtrl);

}());