(function () {
  'use strict';

  function filterCtrl($scope, $state, $ionicLoading, moment, MetaVera, sf, $cordovaDatePicker, $ionicScrollDelegate, $stateParams, $cordovaGeolocation, DriftData, VehiclesType, $timeout){
    var vm = this;

    vm.date = function(usescope){
      return sf.unixType(DriftData.time[usescope], 'DD.MM.YYYY');
    };

    vm.time = function(usescope){
      return sf.unixType(DriftData.time[usescope], 'HH:mm');
    };

    vm.changeTime = function(usescope, picker) {
      var options = {
        date: new Date(sf.unixType(DriftData.time[usescope])),
        mode: picker
      };

      $cordovaDatePicker.show(options).then(function(data){
        var time = (picker == 'date') ?
            sf.unixType(DriftData.time[usescope], 'HH:mm') :
            sf.timeType(data, 'HH:mm');

        var date = (picker == 'time') ?
            sf.unixType(DriftData.time[usescope], 'YYYY-MM-DD') :
            sf.timeType(data, 'YYYY-MM-DD');

        var setTime = moment(date + 'T' + sf.timeRound(time) + ':00').unix();

        if( date >= sf.serverDate() && time >= sf.serverHour() || date > sf.serverDate() && time <= sf.serverHour() ){
          usescope == 'start' ? DriftData.time.start = setTime : DriftData.time.end = setTime;
          vm.filterResult();
        } else {
          sf.alert(sf.translate('res.popup.alert.reserve'));
        }
      });
    };

    vm.filterData = function() {
      return $cordovaGeolocation.getCurrentPosition({}).then(function (position) {
        return MetaVera.doRequest('resultsFromStackFilter', {
          'aStackFilter[vehicleTypeIds][]': $stateParams.car != undefined ? $stateParams.car : [],
          'aStackFilter[startTime]': DriftData.time.start,
          'aStackFilter[endTime]': DriftData.time.end,
          'aStackFilter[latitude]': $stateParams.lat ? $stateParams.lat : position.coords.latitude,
          'aStackFilter[longitude]': $stateParams.long ? $stateParams.long : position.coords.longitude,
          includeStack: 'true'
        }).then(function (data) {
          if (data) {
            // Reject bad cars and sort by distance from the user

            var cars = _.sortBy(
                _.reject(data.DBRankedStacks, function (a) {
                  return a.rank == 'bad';
                }),
                function (e) {
                  return Number(e.distance);
                }
            );

            //Added self location to drift factory
            DriftData.myLocation.lat = $stateParams.lat ? $stateParams.lat : position.coords.latitude;
            DriftData.myLocation.long = $stateParams.long ? $stateParams.long : position.coords.longitude;

            if(cars.length){
              return cars;
            } else {
              $scope.notFound = sf.translate('filter.not_found');
            }
          }
          // or return undefined
        }, function (error) {
          console.log(error);
        });
      }, function (error){
        console.log(error);
      });
    };

    vm.filterResult = function() {
      $scope.notFound = '';
      $scope.message = $scope.items ? sf.translate('filter.update') : sf.translate('filter.search');
      vm.filterData().then(function(cars){
        if(DriftData.time.start < DriftData.time.end) {
          $timeout(function(){
            $ionicScrollDelegate.scrollTo(0, 350, true);
          }, 1500);
          $scope.items = cars;
        } else {
          sf.alert(sf.translate('res.popup.alert.end'));
        }
        $scope.message = '';
      });
    };

    vm.refreshFilter = function(){
      vm.filterData().then(function (cars) {
        $ionicScrollDelegate.scrollTo(0, 480, true);
        $scope.items = cars;
      }).finally(function () {
        $scope.$broadcast('scroll.refreshComplete');
      });
    };

    vm.startFrom = function(time){
      return  sf.translate('filter.ordered') + ' ' + sf.unixType(time, 'HH:mm DD/MM');
    };

    vm.goToCar = function(id, start, end, lat, long){
      $state.go('my.newreservation', {
        id: id,
        start: start,
        end: end,
        lat: lat,
        long: long
      }) ;
    };

    function getIds() {
      var filterResults = $scope.items;
      var items = [];
      for (var i = 0; i < filterResults.length; i++) {
        items.unshift(filterResults[i].DBEntityStack.DBEntityVehicle.id);
      }
      return items;
    }

    vm.goToMap = function() {
      $state.go('mapByVehicles', {
        car: getIds(),
        start: DriftData.time.start,
        end: DriftData.time.end,
        lat: $stateParams.lat,
        long: $stateParams.long,
        name: $stateParams.name,
        type: $stateParams.car
      }) ;
    };

    vm.goTo = function(link) {
      $state.go(link, {
        car: $stateParams.car,
        lat: $stateParams.lat,
        long: $stateParams.long,
        name: $stateParams.name,
        start: DriftData.time.start,
        end: DriftData.time.end
      });
    };

    $scope.carImg = function(res) {
      return sf.getCarImage(res)
    };

    $scope.placename = sf.translate('filter.placename');
    if($stateParams.name) {
      $scope.placename = $stateParams.name;
    }

    $scope.carfilter = sf.translate('filter.carfilter');
    if($stateParams.car) {
      if(typeof $stateParams.car == 'object') {
        $scope.carfilter = $stateParams.car.length + ' ' + sf.translate('filter.cars_c');
      } else {
        _.find(VehiclesType, function(vt){
          $stateParams.car == vt.id ? $scope.carfilter = vt.descr : '';
        });
      }
    }

    $scope.dist = function(distance){
      var km = '';
      var dist = ((distance * 0.001).toFixed(3)).toString().split('.');
      if(!dist[0].indexOf('0')){
        km = Math.round(dist[1]) + ' м';
      } else {
        km = dist[0] + ' км. ' + Math.round(dist[1]) + ' м';
      }
      return km;
    };

    $scope.price = function(id){
      var price = sf.getCarPrice(id) * 4;
      return price + ' руб.';
    };

    vm.filterResult();

  }

  function filterCarsCtrl($scope, $state, $stateParams, VehiclesType) {
    var vm = this;

    $scope.cars = VehiclesType;
    $scope.IDs = [];
    vm.allcars = true;

    vm.clearCars = function(){
      _.find(document.getElementsByTagName('input'), function(a){
        $scope.IDs = [];
        !a.name ? a.checked = false : '';
      });
    };

    vm.getCar = function (id) {
      vm.allcars = false;
      if($scope.IDs.indexOf(id) == -1) {
        $scope.IDs.push(id);
      } else {
        var indexId = $scope.IDs.indexOf(id);
        $scope.IDs.splice(indexId, 1);
      }
    };

    vm.goToFilter = function() {
      $state.go('my.filter', {
        car: $scope.IDs,
        lat: $stateParams.lat,
        long: $stateParams.long,
        name: $stateParams.name
      })
    };

  }

  function filterPlacesCtrl($scope, $state, $http, $stateParams, $translate, $localStorage) {
    var vm = this;
    var yaUrl = 'http://geocode-maps.yandex.ru/1.x/';
    $scope.place = null;
    $scope.history = $localStorage.getObject('location').length ? $localStorage.getObject('location') : [];

    vm.enterPlace = function(place){
      var req = {
        params: {
          geocode: place,
          format: 'json',
          results: 10,
          ll: '30.313497,59.938531',
          lang: $translate.use()
        }
      };

      if(place != '') {
        return $http.get(yaUrl, req, {cache: true}).then(function (locations) {
          $scope.placeRes = locations.data.response.GeoObjectCollection.featureMember;
        });
      }
    };

    $scope.clearPlace = function(){
      document.getElementById('place').value = null;
      $scope.place = null;
      $scope.placeRes = null;
    };

    vm.ownPlace = function(){
      $state.go('my.filter', {
        car: $stateParams.car
      });
    };

    vm.choosePlace = function(pos, name) {
      var coords = pos.split(' ');
      var obj = {pos: {lat: coords[1], long: coords[0]}, name: name};

      if($scope.history.length < 10) {
        $scope.history.push(obj);
        $localStorage.setObject('location', _.uniq($scope.history, 'name'));
      } else {
        $scope.history.shift();
        $scope.history.push(obj);
        $localStorage.setObject('location', _.uniq($scope.history, 'name'));
      }

      $state.go('my.filter', {
        car: $stateParams.car,
        lat: coords[1],
        long: coords[0],
        name: name
      });
    };

    vm.chooseHistory = function(lat, long, name){
      $state.go('my.filter', {
        car: $stateParams.car,
        lat: lat,
        long: long,
        name: name
      });
    };

  }

  function newReservationCtrl($scope, $stateParams, $state, $ionicPopup, $ionicLoading, $ionicHistory, moment, Vehicle, MetaVera, sf, $cordovaDatePicker, DriftData, $timeout) {
    var vm = this;

    vm.Vehicles = Vehicle;
    $scope.vehicles = Vehicle.DBRankedStacks;

    var catchVehicle = _.find($scope.vehicles, function (car) {
      return car.DBEntityStack.DBEntityVehicle.id == $stateParams['id'];
    });

    $scope.vehicle = catchVehicle;
    console.log(catchVehicle);

    //Adding data to drift factory
    DriftData.parking.pDescr = catchVehicle.DBEntityStack.locationDescription;
    DriftData.parking.pPlace = catchVehicle.DBEntityStack.lotDescription;
    DriftData.carLocation.lat = catchVehicle.DBEntityStack.latitude;
    DriftData.carLocation.long = catchVehicle.DBEntityStack.longitude;

    if(!$scope.vehicle){
      $ionicHistory.nextViewOptions({
        disableBack: true,
        historyRoot: true
      });

      $ionicLoading.hide();
      $ionicPopup.alert({
        title: sf.translate('res.popup.attention'),
        template: sf.translate('res.popup.alert.blocked'),
        buttons: [
          {
            text: '<b>Назад</b>',
            type: 'button-positive',
            onTap: function () {
            }
          }
        ]
      }).then(function() {
        console.log($ionicHistory.viewHistory());
        $ionicHistory.goBack();
      });
    }

    if(DriftData.time.start && DriftData.time.end && $scope.vehicle.rank != 'maybe'){
      $scope.startTime = DriftData.time.start;
      $scope.endTime = DriftData.time.end;
    } else {
      if ($scope.vehicle) {
        $scope.startTime = $scope.vehicle.bestStartStamp;
        $scope.endTime = $scope.vehicle.bestEndStamp;
      }
    }

    vm.badRank = function () {
      if ($scope.vehicle.rank == 'bad') {
        $scope.badRank = sf.translate('res.popup.alert.denied');
      }
      return $scope.badRank;
    };

    vm.date = function(usescope){
      return sf.unixType($scope[usescope], 'DD.MM.YYYY');
    };

    vm.time = function(usescope){
      return sf.unixType($scope[usescope], 'HH:mm');
    };

    vm.changeTime = function(usescope, picker) {
      var options = {
        date: new Date(sf.unixType($scope[usescope])),
        mode: picker
      };

      $cordovaDatePicker.show(options).then(function(data){
        var time = (picker == 'date') ?
            sf.unixType($scope[usescope], 'HH:mm') :
            sf.timeType(data, 'HH:mm');

        var date = (picker == 'time') ?
            sf.unixType($scope[usescope], 'YYYY-MM-DD') :
            sf.timeType(data, 'YYYY-MM-DD');

        var setTime = moment(date + 'T' + sf.timeRound(time) + ':00').unix();
        if( date >= sf.serverDate() && time >= sf.serverHour() || date > sf.serverDate() && time <= sf.serverHour() ) {
          $ionicLoading.show();
          usescope == 'startTime' ? $scope.startTime = setTime : $scope.endTime = setTime;
          $scope.timeChanged = vm.travelCurrency();
        } else {
          $ionicLoading.hide();
          sf.alert(sf.translate('res.popup.alert.reserve'));
        }
      });
    };

    vm.travelCurrency = function () {
      MetaVera.tripEstimate($scope.vehicle.DBEntityStack.id, $scope.startTime, $scope.endTime).then(function(data){
        $scope.price = sf.translate('res.t_cost') + ' ' + data.replace('руб. ','') + ' ' + sf.translate('res.currency');
        $ionicLoading.hide();
      }, function(error){
        $ionicLoading.hide();
        sf.alert(error.string);
      });
    };

    vm.makeReservation = function () {
      $ionicLoading.show();
      MetaVera.makeReservationGetId($scope.vehicle.DBEntityStack.id, $scope.startTime, $scope.endTime).then(function(data){
        if (data) {
          sf.addNot(data, $scope.startTime, 3600, 'start');
          $timeout(function(){
            sf.addNot(data, $scope.endTime, 1800, 'end');
          }, 1000);
          $ionicLoading.hide();
          $scope.reserved = sf.translate('res.popup.alert.reserved');
          $scope.$emit('my.reservations'); $state.go('my.reservations', {}, {reload: true});
        }
      }, function(error) {
        if(error.string != '[object Object]') {
          $ionicLoading.hide();
          sf.alert(error.string);
        } else {
          $ionicLoading.hide();
          sf.alert(sf.translate('res.popup.alert.error_s'));
        }
      });
    };

    $scope.carImg = function(res) {
      return sf.getCarImage(res)
    };

    $scope.goPrice = function (id) {
      return sf.goPricingPage(id);
    };

  }

  function parkingCtrl($scope, DriftData) {
    var vm = this;
    $scope.parking = DriftData.parking;

    vm.openNavi = function() {
      window.open('https://maps.yandex.ru/?text='+ DriftData.carLocation.lat +'%20'+ DriftData.carLocation.long +'&rtext='+ DriftData.myLocation.lat +'%2C'+ DriftData.myLocation.long +'~'+ DriftData.carLocation.lat +'%2C'+ DriftData.carLocation.long +'', '_system', 'location=yes');
    };
  }

  angular
      .module('app.newreservation')
      .controller('filterCtrl', filterCtrl)
      .controller('filterCarsCtrl', filterCarsCtrl)
      .controller('filterPlacesCtrl', filterPlacesCtrl)
      .controller('newReservationCtrl', newReservationCtrl)
      .controller('parkingCtrl', parkingCtrl);
}());