(function () {
  'use strict';

  function ReservationCtrl($scope, $cordovaDatePicker, $timeout, $state, $interval, $ionicPopup, $ionicLoading, moment, Reservation, MetaVera, $localStorage, Cars, sf, $cordovaLocalNotification, DriftData, Position) {
    var vm = this;
    vm.Reservation = Reservation;
    $scope.reservation = Reservation.DBEntityReservation;

    $scope.newStart = $scope.reservation.startStamp;
    $scope.newEnd = $scope.reservation.endStamp;

    console.log(Reservation);

    //Adding data to drift factory
    DriftData.parking.pDescr = $scope.reservation.DBEntityStack.locationDescription;
    DriftData.parking.pPlace = $scope.reservation.DBEntityStack.lotDescription;
    DriftData.carLocation.lat = $scope.reservation.DBEntityStack.latitude;
    DriftData.carLocation.long = $scope.reservation.DBEntityStack.longitude;
    DriftData.myLocation.lat = Position.coords.latitude;
    DriftData.myLocation.long = Position.coords.longitude;

    vm.saveSwipeInStatus = function (r_id, swipe_id, swipe_status) {
      var curObject = $localStorage.getObject('reservation_' + r_id);
      curObject['swipeIn'] = {id: swipe_id, status: swipe_status};
      curObject['swipeOut'] = {id: '', status: ''};
      $localStorage.setObject('reservation_' + r_id, curObject);
    };

    vm.saveSwipeOutStatus = function (r_id, swipe_id, swipe_status) {
      var curObject = $localStorage.getObject('reservation_' + r_id);
      curObject['swipeOut'] = {id: swipe_id, status: swipe_status};
      curObject['swipeIn'] = {id: '', status: ''};
      $localStorage.setObject('reservation_' + r_id, curObject);
    };

    vm.restoreSwipeState = function (r_id) {
      var curObject = $localStorage.getObject('reservation_' + r_id);
      if (curObject) {
        if (curObject['swipeIn']) {
          $scope.swipeId = curObject['swipeIn'].id;
          $scope.swipeStatus = curObject['swipeIn'].status;
        }

        if (curObject['swipeOut']) {
          $scope.swipeOutId = curObject['swipeOut'].id;
          $scope.swipeOutStatus = curObject['swipeOut'].status;
        }
      }
    };

    vm.reservationInProgress = function(r) {
      return moment().unix() >= Number(r.startStamp) && moment().unix() <= Number(r.endStamp);
    };

    vm.reservationCompleted = function(r) {
      return moment().unix() >= Number(r.startStamp) && moment().unix() >= Number(r.endStamp);
    };

    vm.refreshSwipeStatus = function(d) {
      return MetaVera.getSwipeInStatus(d);
    };

    vm.refreshSwipeInStatus = function() {
      return this.refreshSwipeStatus($scope.swipeId).then(function (a) {
        $scope.swipeStatus = a;

        vm.saveSwipeInStatus(vm.Reservation.DBEntityReservation.id, $scope.swipeId, a);
      })
    };

    vm.refreshSwipeOutStatus = function () {
      return this.refreshSwipeStatus($scope.swipeOutId).then(function (a) {
        $scope.swipeOutStatus = a;
        vm.saveSwipeOutStatus(vm.Reservation.DBEntityReservation.id, $scope.swipeOutId, a);
      })
    };

    vm.cancelReservation = function(reservation) {
      $ionicPopup.show({
        title: sf.translate('res.popup.attention'),
        template: sf.translate('res.popup.alert.sure'),
        buttons: [
          {text: sf.translate('res.popup.no'), type: 'button-assertive'},
          {
            text: '<b>'+ sf.translate('res.popup.yes') +'</b>',
            type: 'button-positive',
            onTap: function () {
              $ionicLoading.show();
              MetaVera.cancelReservationWithId(reservation.id).then(function () {
                vm.confirmCancel();
                vm.cancelNot(reservation);
              });
            }
          }
        ]
      });
    };

    vm.confirmCancel = function() {
      $ionicLoading.hide();
      $ionicPopup.alert({
        title: sf.translate('res.popup.attention'),
        template: sf.translate('res.popup.finish.cancel'),
        scope: $scope,
        buttons: [
          {
            text: '<b>OK</b>',
            type: 'button-positive',
            onTap: function () {
              $scope.reservationClosed = 'Reservation have been closed.';
              $scope.$emit('my.reservations'); $state.go('my.reservations', {}, {reload: true});
            }
          }
        ]
      });
    };

    vm.cancelNot = function(reservation){
      if(ionic.Platform.isWebView()){
        $cordovaLocalNotification.cancel([reservation.id, reservation.id + 1]).then(function(){
          console.log('All notifications have been canceled.');
        });
      }
    };

    vm.date = function(date) {
      return sf.unixType(date, 'DD.MM.YYYY');
    };

    vm.time = function(time){
      return sf.unixType(time, 'HH:mm');
    };

    vm.changeTime = function(usescope, picker) {
      var options = {
        date: new Date(sf.unixType($scope[usescope])),
        mode: picker
      };

      $cordovaDatePicker.show(options).then(function(data){
        var time = (picker == 'date') ? // if
            sf.unixType($scope[usescope], 'HH:mm') : // else
            sf.timeType(data, 'HH:mm');

        var date = (picker == 'time') ? // if
            sf.unixType($scope[usescope], 'YYYY-MM-DD') : // else
            sf.timeType(data, 'YYYY-MM-DD');

        var setTime = moment(date + 'T' + sf.timeRound(time) + ':00').unix();
        if( date >= sf.serverDate() && time >= sf.serverHour() || date > sf.serverDate() && time <= sf.serverHour() ) {
          return usescope == 'newStart' ?
              $scope.newStart = setTime :
              $scope.newEnd = setTime;
        } else {
          vm.extendTimeError();
        }
      });
    };

    //Edit of future reservation
    vm.editRes = function(reservation){
      $scope.start = 'newStart';
      $scope.end = 'newEnd';
      $ionicPopup.show({
        title: sf.translate('res.popup.edit.title'),
        templateUrl: 'templates/reservation/popup/edit.html',
        scope: $scope,
        buttons: [
          {
            text: sf.translate('res.popup.cancel'),
            type: 'button-assertive',
            onTap: function() {
              $scope.newStart = reservation.startStamp;
              $scope.newEnd = reservation.endStamp;
            }
          },
          {
            text: '<b>'+ sf.translate('res.popup.edit.change') +'</b>',
            type: 'button-positive',
            onTap: function() {
              $ionicLoading.show();
              vm.editReservationTimes(reservation, $scope.newStart, $scope.newEnd)
            }
          }
        ]
      });
    };

    vm.editReservationTimes = function(res, start, end) {
      MetaVera.editReservationTimes(res.id, start, end).then(function (data) {
        if (data) {
          vm.cancelNot(res);
          res.startStamp = start; res.endStamp = end;
          sf.addNot(res.id, start, 3600, 'start');
          $timeout(function(){
            sf.addNot(res.id, end, 1800, 'end');
          }, 1000);
          $ionicLoading.hide();
        }
      }, function (error) {
        vm.extendTimeClosed(error);
      });
    };

    //Extend of current reservation
    vm.extendRes = function(reservation){
      $scope.end = 'newEnd';
      $ionicPopup.show({
        title: sf.translate('res.popup.extend.title'),
        templateUrl: 'templates/reservation/popup/extend.html',
        scope: $scope,
        buttons: [
          {
            text: sf.translate('res.popup.cancel'),
            type: 'button-assertive',
            onTap: function() {
              $scope.newEnd = reservation.endStamp;
            }
          },
          {
            text: '<b>'+ sf.translate('res.popup.extend.extend') +'</b>',
            type: 'button-positive',
            onTap: function() {
              $ionicLoading.show();
              vm.extendReservationWithId(reservation, $scope.newEnd)
            }
          }
        ]
      });
    };

    vm.extendReservationWithId = function(res, end) {
      MetaVera.extendReservationWithId(res.id, end).then(function (data) {
        if (data) {
          vm.cancelNot(res);
          res.endStamp = end;
          sf.addNot(res.id, end, 1800, 'end');
          $ionicLoading.hide();
        }
      }, function (error) {
        vm.extendTimeError(error);
      });
    };

    //Start trip with MetaVera virtual swipe in
    vm.virtualSwipeIn = function(reservation) {
      var numberOfTries = 0;
      $ionicLoading.show();

      MetaVera.virtualSwipeInOnReservation(reservation.id).then(function (d) {
        $scope.swipeId = d;
        numberOfTries = numberOfTries + 1;

        var inRefreshInterval = $interval(function () {
          vm.refreshSwipeInStatus().then(function () {
            if ($scope.swipeStatus == 'sentack') {
              $ionicLoading.hide();

              $interval.cancel(inRefreshInterval);

            }
          });

        }, 1000, 60);

        if (numberOfTries === 60) {
          $ionicLoading.hide();
        }

      }, function(d) {
        // We understand that the car has already been opened
        if (/has already been started/.test(d.string)) {
          $scope.swipeStatus = 'sentack';
        }
      });
    };

    //End trip with MetaVera virtual swipe in
    vm.virtualSwipeOut = function (reservation) {
      var numberOfTries = 0;
      $ionicLoading.show();

      MetaVera.virtualSwipeOutOnReservation(reservation.id).then(function (d) {
        $scope.swipeOutId = d;
        numberOfTries = numberOfTries + 1;

        var inRefreshInterval2 = $interval(function () {
          vm.refreshSwipeOutStatus().then(function () {
            if ($scope.swipeStatus == 'sentack') {
              $timeout(function() {
                $ionicLoading.hide();
                $interval.cancel(inRefreshInterval2);
                vm.confirmedClosed();
              }, 7000);
            }
          });

          if (numberOfTries === 60) {
            $ionicLoading.hide();
          }
        }, 1000, 60);

        return d;

      }, function(d) {
        // We understand that the car has already been ended
        if (/has already been ended/.test(d.string)) {
          $ionicLoading.hide();
          $scope.swipeOutStatus = 'sentack';
        }
      });
    };

    vm.confirmedClosed = function() {
      $ionicLoading.hide();
      if ( $scope.warningPopup ) {
        $scope.warningPopup.close();
      }

      if (!$scope.alertPopup) {

        $scope.alertPopup = $ionicPopup.alert({
          title: sf.translate('res.popup.attention'),
          template: sf.translate('res.popup.finish.closed')
        }).then(function (popup) {
          $scope.alertPopup = undefined;
        });
        $scope.swipeId = undefined;
        $scope.swipeStatus = undefined;
        $scope.swipeOutId = undefined;
        $scope.swipeOutStatus = undefined;
      }
    };

    vm.confirmedError = function () {
      $ionicLoading.hide();
      if ($scope.warningPopup) {
        $scope.warningPopup.close();
      }
      $ionicPopup.alert({
        title: sf.translate('res.popup.attention'),
        template: sf.translate('res.popup.alert.error')
      });
      $scope.swipeId = undefined;
      $scope.swipeStatus = undefined;
      $scope.swipeOutId = undefined;
      $scope.swipeOutStatus = undefined;
    };

    vm.extendTimeError = function () {
      $ionicLoading.hide();
      if ($scope.warningPopup) {
        $scope.warningPopup.close();
      }
      sf.alert(sf.translate('res.popup.alert.extend'));
    };

    vm.extendTimeClosed = function (error) {
      $ionicLoading.hide();
      if ($scope.warningPopup) {
        $scope.warningPopup.close();
      }
      sf.alert(error.string);
    };

    vm.openCloseModal = function (reservation) {
      $scope.warningPopup = $ionicPopup.show({
        title: sf.translate('res.popup.attention'),
        template: sf.translate('res.popup.finish.check'),
        buttons: [
          {
            text: sf.translate('res.popup.cancel'),
            type: 'button-balanced',
            onTap: function (e) {
            }
          },
          {
            text: 'OK',
            type: 'button-assertive',
            onTap: function (e) {
              vm.virtualSwipeOut(reservation);
              e.preventDefault();
            }
          }
        ]
      });
    };

    $scope.moment = moment;

    $scope.cars = Cars;

    $scope.getCarImage = function (res) {
      try {
        return Cars[res.DBEntityStack.DBEntityVehicleType.id]['img'];
      } catch (E) {

      }

      try {
        return res.DBEntityStack.DBEntityVehicleType.thumbDesc;
      } catch (E) {

      }
      return '';

    };

    $scope.goPricingPage = function (res) {
      $state.go('carPricing', {id: res.DBEntityStack.DBEntityVehicleType.id});
    };

    vm.restoreSwipeState($scope.reservation.id);

  }


  function ReservationsListCtrl($scope, SessionData, PastData, User, sf, $rootScope) {

    var vm = this;

    console.log(SessionData);

    vm.currentRes = SessionData.WSGetCurrentAndFutureReservationsResult.DBEntityReservation;
    vm.Reservations = SessionData.WSGetCurrentAndFutureReservationsResult;

    $scope.currentRes = SessionData.WSGetCurrentAndFutureReservationsResult.DBEntityReservation;
    $scope.Reservations = SessionData.WSGetCurrentAndFutureReservationsResult;
    if ( PastData.DBEntityReservation) {
      var total = PastData.DBEntityReservation.length;
      if (total){
        $scope.pastRes = PastData.DBEntityReservation.slice(total - 10);
      } else {
        $scope.pastRes = PastData.DBEntityReservation;
      }
    }

    vm.time = function(time){
      return sf.unixType(time, 'DD.MM.YYYY '+ sf.translate('res.in') +' HH:mm');
    };

    $rootScope.$on('my.reservations', function() {
      vm.doRefresh();
    });

    vm.doRefresh = function () {
      User.fetchSessionData().then(function (sessiondata) {
        $scope.Reservations = sessiondata.WSGetCurrentAndFutureReservationsResult;
        $scope.currentRes = sessiondata.WSGetCurrentAndFutureReservationsResult.DBEntityReservation;
      }).finally(function () {
        // Stop the ion-refresher from spinning
        $scope.$broadcast('scroll.refreshComplete');
      });
    };

  }

  angular
      .module('app.reservations')
      .controller('ReservationsListCtrl', ReservationsListCtrl)
      .controller('ReservationCtrl', ReservationCtrl);
}());